use strict;
use DB_File::Lock;
chdir '/home/kleptog/perl/ddtss' or die;
my $db_hash = "ddtss.db";

my %hash;

# Open DB for write mode
sub DDTSS_Open_Write
{
  my $db = tie %hash,  'DB_File::Lock', $db_hash, O_RDWR, 0666, $DB_HASH, "write";
  if( not defined $db )
  { die "Unable to open DB file for writing ($!)\n" }
  return $db;
}

# Open DB for read mode
sub DDTSS_Open_Read
{
  my $db = tie %hash,  'DB_File::Lock', $db_hash, O_RDONLY, 0666, $DB_HASH, "read";
  if( not defined $db )
  { die "Unable to open DB file for reading ($!)\n" }
  return $db;
}

# close DB, undef caller reference
sub DDTSS_Close
{
  undef $_[0];
  untie %hash;
}

# Get list of untranslated packages
sub DDTSS_Get_Untranslated
{
  my $db = shift;
  my ($key,$value);
  my @todo;
  for (my $status = $db->seq($key, $value, R_FIRST) ;
       $status == 0 ;
       $status = $db->seq($key, $value, R_NEXT) )
  {
    # $1 = prio, $2 = package
    next if $key !~ m,^untranslated/(\d+)/(.*),;

    push @todo, [$1,$2];
  }
  return [ map { $_->[1] } sort { $a->[0] <=> $b->[0] } @todo ];
}

# Get list of packages for review
sub DDTSS_Get_ForReview
{
  my $db = shift;
  my $user = shift;
  
  my ($key,$value);
  my @todo;
  for (my $status = $db->seq($key, $value, R_FIRST) ;
       $status == 0 ;
       $status = $db->seq($key, $value, R_NEXT) )
  {
    # $1 = package
    next if $key !~ m,^forreview/([^/]+)/owner,;
    my $package = $1;
    my $str = "";
    if( $value eq $user )
    { $str = " (owner)" }
    else
    {
      my $reviewers;
      if( $db->get("forreview/$package/reviewers", $reviewers) == 0 )
      {
        if( scalar( grep { $_ eq $user } split /,/, $reviewers ) )
        { $str = " (reviewed)" }
      }
    }
    push @todo, [$package,$str];
  }
  return [ sort { $a cmp $b } @todo ];
}

# Given a package name, find the key in the database that represents the untranslated description
sub DDTSS_Untranslated_Get_Key
{
  my $db = shift;
  my $package = shift;

  my ($key,$value);
  for (my $status = $db->seq($key, $value, R_FIRST) ;
       $status == 0 ;
       $status = $db->seq($key, $value, R_NEXT) )
  {
    return $key if $key =~ m,^untranslated/\d+/$package,;
  }
  return undef;
}

# Does a sort of soft-lock on a key
sub DDTSS_TryLock
{
  my $db = shift;
  my $key = shift;
  my $user = shift;
  
  my $field;  
  my($lockuser,$locktime);
  if( $db->get( "lock/$key", $field ) )
  { $db->del( "lock/$key" ); $lockuser = ""; $locktime = 0 }
  else
  {
    ($lockuser,$locktime) = split /,/, $field;
  }
  
  if( ($locktime > time() - 900) and ($lockuser ne $user) )
  {
    return 1;
  }
  # May update the time
  $db->put( "lock/$key", $user.",".time() );
  return 0;
}

# Release lock
sub DDTSS_Unlock
{
  my $db = shift;
  my $key = shift;
  my $user = shift;
  
  my $field;
  if( $db->get( "lock/$key", $field ) == 0 )
  {
    my ($lockuser,$locktime) = split /,/, $field;
    if( $lockuser eq $user )
    {
      $db->del( "lock/$key" );
    }
  }
}

# When a new description comes in, we delete the oldest request
sub DDTSS_Kill_Oldest_Request
{
  my $db = shift;
  my $oldest = 2**32;

  my($key,$value);
  
  for (my $status = $db->seq($key, $value, R_FIRST) ;
       $status == 0 ;
       $status = $db->seq($key, $value, R_NEXT) )
  {
    # Found a sent request
    if( $key =~ m,^requests/, )
    {
      if( $value < $oldest )
      { $oldest = $value }
    }
  }
  
  $db->del( "requests/$oldest" );
  return;
}

sub DDTSS_Load_Wordlist
{
  my $lang = shift;
  my $fh;
  open $fh, "words-$lang.txt" or return undef;
  
  my $wordlist = {};
  
  while(<$fh>)
  {
    my @f = split /\t/, $_, 2;
    $wordlist->{$f[0]} = $f[1];
  }
  
  return $wordlist;
}
1;